import os
import time
import typing

import configargparse


def get_hosts(options: configargparse.Namespace) -> typing.List[str]:
    host_option: typing.List[str] = options.host
    if host_option:
        return host_option
    hosts_option = options.hosts.split(',')
    if len(hosts_option) == 1 and hosts_option[0] == '':
        return []
    return hosts_option


def startup_sleep():
    secs = int(os.getenv('STARTUP_SLEEPING_SECONDS', '3'))
    time.sleep(secs=secs)


def parse() -> configargparse.Namespace:
    p = configargparse.ArgParser()
    p.add('-c', '--config', required=False, is_config_file=True, help='config file path')
    p.add('--host', action='append', default=[], help='remote host address, simulate training if not set')
    p.add(
        '--hosts',
        default='',
        help='remote host address, simulate training if not set, seperated by commas',
        env_var='WOKER_HOSTS',
    )
    p.add('--ca-cert', default='', help='CA cert path')
    return p.parse_args()


if __name__ == '__main__':
    options = parse()
    print(options)



import typing

from . import configuration, connection, k8s

__version__ = '0.0.7'


def initialize() -> None:
    options = configuration.parse()
    hosts: typing.List[str] = configuration.get_hosts(options=options)
    ca_cert_path: str = options.ca_cert

    if not hosts:
        return
    k8s.notify_istio_proxy_quit_before_exit()
    configuration.startup_sleep()
    connection.initialize_grpc(hosts=hosts, ca_cert_path=ca_cert_path)



import logging

import grpc
import tensorflow_federated as tff


def initialize_grpc(hosts, ca_cert_path: str) -> None:
    if ca_cert_path:
        logging.debug(f'using CA cert located at {ca_cert_path}')
        with open(ca_cert_path, 'rb') as f:
            creds = grpc.ssl_channel_credentials(f.read())
        channels = [grpc.secure_channel(host, creds) for host in hosts]
    else:
        logging.debug('not using CA cert')
        channels = [grpc.insecure_channel(host) for host in hosts]

    tff.backends.native.set_remote_execution_context(channels)


if __name__ == '__main__':
    initialize_grpc(hosts=[], ca_cert_path='')



import atexit
import logging
from urllib import error, request

logger = logging.getLogger(__name__)


def notify_istio_proxy_quit():
    path = 'http://127.0.0.1:15020/quitquitquit'
    try:
        request.urlopen(request.Request(path, data=''.encode()))
    except error.URLError as e:
        logger.error(f'Quit Istio error: {e}')


def notify_istio_proxy_quit_before_exit():
    atexit.register(notify_istio_proxy_quit)
